@@ -10,13 +10,14 @@ from .common import *
 from .contentAPI import ContentAPI
 from .entityAPI import EntityAPI
 from .uploadAPI import UploadAPI, simple_asset_package, complex_asset_package, cvs_to_xsd, cvs_to_xml, \
-                    cvs_to_cmis_xslt, csv_to_search_xml
-from .workflowAPI import WorkflowAPI
-from .retentionAPI import RetentionAPI
+                    cvs_to_cmis_xslt, csv_to_search_xml, generic_asset_package, upload_config
+from .workflowAPI import WorkflowAPI, WorkflowContext, WorkflowInstance
+from .retentionAPI import RetentionAPI, RetentionAssignment, RetentionPolicy
+from .parAPI import PreservationActionRegistry
 
 __author__ = "James Carr (drjamescarr@gmail.com)"
 
 # Version of the Preservica API package
-__version__ = "0.9.8"
+__version__ = "1.0.2"
 
 __license__ = "Apache License Version 2.0"
