@@ -1,3 +1,15 @@
+"""
+pyPreservica EntityAPI module definition
+
+A client library for the Preservica Repository web services Entity API
+https://us.preservica.com/api/entity/documentation.html
+
+author:     James Carr
+licence:    Apache License 2.0
+
+"""
+
+
 import xml.etree.ElementTree
 from typing import Set
 
@@ -43,17 +55,24 @@ class RetentionPolicy:
 
 class RetentionAPI(AuthenticatedAPI):
 
-    def __init__(self, username=None, password=None, tenant=None, server=None, use_shared_secret=False):
+    def __init__(self, username=None, password=None, tenant="%", server=None, use_shared_secret=False):
         super().__init__(username, password, tenant, server, use_shared_secret)
         if self.major_version < 7 and self.minor_version < 2:
             raise RuntimeError("Retention API is only available when connected to a v6.2 System")
 
     def policy(self, reference: str) -> RetentionPolicy:
         """
-           Return a retention policy by its reference
-        """
+         Return a retention policy by reference
+
+        :param reference: The policy reference
+        :type reference: str
+
+        :return: The retention policy
+        :rtype: RetentionPolicy
+
+         """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/xml;charset=UTF-8'}
-        request = requests.get(f'https://{self.server}/api/entity/retention-policies/{reference}', headers=headers)
+        request = self.session.get(f'https://{self.server}/api/entity/retention-policies/{reference}', headers=headers)
         if request.status_code == requests.codes.ok:
             xml_response = str(request.content.decode('utf-8'))
             logger.debug(xml_response)
@@ -92,10 +111,22 @@ class RetentionAPI(AuthenticatedAPI):
             raise RuntimeError(request.status_code, "policy failed")
 
     def assignable_policy(self, reference: str, status: bool):
+        """
+        Make a policy assignable
+
+        :param reference:  The policy ID
+        :type reference: str
+
+        :param status:     The assignable status
+        :type status: bool
+
+
+        :return:
+        """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'text/plain;charset=UTF-8'}
         data = str(status)
-        request = requests.put(f'https://{self.server}/api/entity/retention-policies/{reference}/assignable',
-                               headers=headers, data=data)
+        request = self.session.put(f'https://{self.server}/api/entity/retention-policies/{reference}/assignable',
+                                   headers=headers, data=data)
         if request.status_code == requests.codes.ok:
             pass
         elif request.status_code == requests.codes.unauthorized:
@@ -190,8 +221,8 @@ class RetentionAPI(AuthenticatedAPI):
 
         xml_request = xml.etree.ElementTree.tostring(retention_policy, encoding='utf-8')
 
-        request = requests.put(f'https://{self.server}/api/entity/retention-policies', data=xml_request,
-                               headers=headers)
+        request = self.session.put(f'https://{self.server}/api/entity/retention-policies', data=xml_request,
+                                   headers=headers)
         if request.status_code == requests.codes.ok:
             return self.policy(reference)
         elif request.status_code == requests.codes.unauthorized:
@@ -285,8 +316,8 @@ class RetentionAPI(AuthenticatedAPI):
 
         xml_request = xml.etree.ElementTree.tostring(retention_policy, encoding='utf-8')
 
-        request = requests.post(f'https://{self.server}/api/entity/retention-policies', data=xml_request,
-                                headers=headers)
+        request = self.session.post(f'https://{self.server}/api/entity/retention-policies', data=xml_request,
+                                    headers=headers)
         if request.status_code == requests.codes.ok:
             xml_response = str(request.content.decode('utf-8'))
             entity_response = xml.etree.ElementTree.fromstring(xml_response)
@@ -304,9 +335,14 @@ class RetentionAPI(AuthenticatedAPI):
     def delete_policy(self, reference: str):
         """
         Delete a retention policy
+
+        :param reference: The policy reference
+        :type reference: str
+
         """
         headers = {HEADER_TOKEN: self.token}
-        request = requests.delete(f'https://{self.server}/api/entity/retention-policies/{reference}', headers=headers)
+        request = self.session.delete(f'https://{self.server}/api/entity/retention-policies/{reference}',
+                                      headers=headers)
         if request.status_code == requests.codes.no_content:
             pass
         elif request.status_code == requests.codes.unauthorized:
@@ -319,10 +355,17 @@ class RetentionAPI(AuthenticatedAPI):
     def policy_by_name(self, name: str) -> RetentionPolicy:
         """
          Return a retention policy by name
+
+        :param name: The policy name
+        :type name: str
+
+        :return: The retention policy
+        :rtype: RetentionPolicy
+
          """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/xml;charset=UTF-8'}
         data = {'start': str(0), 'max': "250"}
-        request = requests.get(f'https://{self.server}/api/entity/retention-policies', data=data, headers=headers)
+        request = self.session.get(f'https://{self.server}/api/entity/retention-policies', data=data, headers=headers)
         if request.status_code == requests.codes.ok:
             xml_response = str(request.content.decode('utf-8'))
             logger.debug(xml_response)
@@ -342,10 +385,15 @@ class RetentionAPI(AuthenticatedAPI):
         """
         Return a list of all retention policies
         Only returns the first 250 policies in the system
+
+
+        :return: Set of retention policies
+        :rtype: Set[RetentionPolicy]
+
         """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/xml;charset=UTF-8'}
         data = {'start': str(0), 'max': "250"}
-        request = requests.get(f'https://{self.server}/api/entity/retention-policies', data=data, headers=headers)
+        request = self.session.get(f'https://{self.server}/api/entity/retention-policies', data=data, headers=headers)
         if request.status_code == requests.codes.ok:
             xml_response = str(request.content.decode('utf-8'))
             entity_response = xml.etree.ElementTree.fromstring(xml_response)
@@ -368,7 +416,16 @@ class RetentionAPI(AuthenticatedAPI):
 
     def add_assignments(self, entity: Entity, policy: RetentionPolicy) -> RetentionAssignment:
         """
-            assign a retention policy to an asset.
+        Assign a retention policy to an Asset.
+
+        :param entity: The Preservica Entity to assign a policy to
+        :type entity: Entity
+
+        :param policy: The RetentionAssignment
+        :type policy: RetentionPolicy
+
+        :return: The RetentionAssignment
+        :rtype: RetentionAssignment
 
         """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/xml;charset=UTF-8'}
@@ -379,7 +436,7 @@ class RetentionAPI(AuthenticatedAPI):
         xml.etree.ElementTree.SubElement(assignment, "RetentionPolicy").text = policy.reference
         xml_request = xml.etree.ElementTree.tostring(assignment, encoding='utf-8').decode('utf-8')
         logger.debug(xml_request)
-        request = requests.post(
+        request = self.session.post(
             f'https://{self.server}/api/entity/{entity.path}/{entity.reference}/retention-assignments',
             headers=headers, data=xml_request)
 
@@ -403,12 +460,20 @@ class RetentionAPI(AuthenticatedAPI):
 
     def remove_assignments(self, retention_assignment: RetentionAssignment):
         """
-           Delete a retention policy from an asset
-         """
+        Delete a retention policy from an asset
+
+        :param retention_assignment: The Preservica Entity to assign a policy to
+        :type retention_assignment: RetentionAssignment
+
+
+        :return: The Asset Reference
+        :rtype: str
+
+        """
 
         headers = {HEADER_TOKEN: self.token}
 
-        request = requests.delete(
+        request = self.session.delete(
             f'https://{self.server}/api/entity/information-objects/{retention_assignment.entity_reference}/retention'
             f'-assignments/{retention_assignment.api_id}', headers=headers)
         if request.status_code == requests.codes.no_content:
@@ -422,9 +487,16 @@ class RetentionAPI(AuthenticatedAPI):
     def assignments(self, entity: Entity) -> Set[RetentionAssignment]:
         """
           Return a list of retention policies for an entity.
+
+          :param entity: The entity to fetch assignments for
+          :type entity: class:`Entity`
+
+          :return: Set of policy assignments
+          :rtype: Set[RetentionAssignment]
+
         """
         headers = {HEADER_TOKEN: self.token, 'Content-Type': 'application/xml;charset=UTF-8'}
-        request = requests.get(
+        request = self.session.get(
             f'https://{self.server}/api/entity/{entity.path}/{entity.reference}/retention-assignments',
             headers=headers)
         if request.status_code == requests.codes.ok:
