@@ -11,16 +11,76 @@ from datetime import datetime
 from xml.dom import minidom
 from xml.etree import ElementTree
 from xml.etree.ElementTree import Element, SubElement
-from boto3.s3.transfer import TransferConfig
+
+import six
+from boto3.s3.transfer import TransferConfig, S3Transfer
 from botocore.config import Config
 from botocore.exceptions import ClientError
 
+import s3transfer.upload
+import s3transfer.tasks
+from s3transfer import S3UploadFailedError
+
 from pyPreservica.common import *
+from pyPreservica.common import _make_stored_zipfile
 
 logger = logging.getLogger(__name__)
 
 GB = 1024 ** 3
-transfer_config = TransferConfig(multipart_threshold=int((1 * GB) / 8))
+transfer_config = TransferConfig(multipart_threshold=int((1 * GB) / 16))
+
+
+def upload_file(self, filename, bucket, key,
+                callback=None, extra_args=None):
+    """Upload a file to an S3 object.
+
+    Variants have also been injected into S3 client, Bucket and Object.
+    You don't have to use S3Transfer.upload_file() directly.
+
+    .. seealso::
+        :py:meth:`S3.Client.upload_file`
+        :py:meth:`S3.Client.upload_fileobj`
+    """
+    if not isinstance(filename, six.string_types):
+        raise ValueError('Filename must be a string')
+
+    subscribers = self._get_subscribers(callback)
+    future = self._manager.upload(
+        filename, bucket, key, extra_args, subscribers)
+    try:
+        return future.result()
+    # If a client error was raised, add the backwards compatibility layer
+    # that raises a S3UploadFailedError. These specific errors were only
+    # ever thrown for upload_parts but now can be thrown for any related
+    # client error.
+    except ClientError as e:
+        raise S3UploadFailedError(
+            "Failed to upload %s to %s: %s" % (
+                filename, '/'.join([bucket, key]), e))
+
+
+class PutObjectTask(s3transfer.tasks.Task):
+    # Copied from s3transfer/upload.py, changed to return the result of client.put_object.
+    def _main(self, client, fileobj, bucket, key, extra_args):
+        with fileobj as body:
+            response = client.put_object(Bucket=bucket, Key=key, Body=body, **extra_args)
+            return response
+
+
+class CompleteMultipartUploadTask(s3transfer.tasks.Task):
+    # Copied from s3transfer/tasks.py, changed to return a result.
+    def _main(self, client, bucket, key, upload_id, parts, extra_args):
+        return client.complete_multipart_upload(
+            Bucket=bucket,
+            Key=key,
+            UploadId=upload_id,
+            MultipartUpload={"Parts": parts},
+            **extra_args,
+        )
+
+
+s3transfer.upload.PutObjectTask = PutObjectTask
+s3transfer.upload.CompleteMultipartUploadTask = CompleteMultipartUploadTask
 
 
 def prettify(elem):
@@ -119,12 +179,23 @@ def __make_bitstream__(xip, file_name, full_path, callback):
     filesize.text = str(file_stats.st_size)
     physical_location = SubElement(bitstream, "PhysicalLocation")
     fixities = SubElement(bitstream, "Fixities")
-    fixity = SubElement(fixities, "Fixity")
-    fixity_algorithm_ref = SubElement(fixity, "FixityAlgorithmRef")
-    fixity_value = SubElement(fixity, "FixityValue")
-    fixity = callback(file_name, full_path)
-    fixity_algorithm_ref.text = fixity[0]
-    fixity_value.text = fixity[1]
+    fixity_result = callback(file_name, full_path)
+    if type(fixity_result) == tuple:
+        fixity = SubElement(fixities, "Fixity")
+        fixity_algorithm_ref = SubElement(fixity, "FixityAlgorithmRef")
+        fixity_value = SubElement(fixity, "FixityValue")
+        fixity_algorithm_ref.text = fixity_result[0]
+        fixity_value.text = fixity_result[1]
+    elif type(fixity_result) == dict:
+        for key, val in fixity_result.items():
+            fixity = SubElement(fixities, "Fixity")
+            fixity_algorithm_ref = SubElement(fixity, "FixityAlgorithmRef")
+            fixity_value = SubElement(fixity, "FixityValue")
+            fixity_algorithm_ref.text = key
+            fixity_value.text = val
+    else:
+        logger.error("Could Not Find Fixity Value")
+        raise RuntimeError("Could Not Find Fixity Value")
 
 
 def __make_representation_multiple_co__(xip, rep_name, rep_type, rep_files, io_ref):
@@ -300,12 +371,12 @@ def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Tit
 
     short_name = "csv"
 
-    xml_schemaName = xml.etree.ElementTree.SubElement(xml_index, "schemaName")
-    xml_schemaName.text = title
-    xml_schemaUri = xml.etree.ElementTree.SubElement(xml_index, "schemaUri")
-    xml_schemaUri.text = xml_namespace
-    xml_shortName = xml.etree.ElementTree.SubElement(xml_index, "shortName")
-    xml_shortName.text = short_name
+    xml_schema_name = xml.etree.ElementTree.SubElement(xml_index, "schemaName")
+    xml_schema_name.text = title
+    xml_schema_uri = xml.etree.ElementTree.SubElement(xml_index, "schemaUri")
+    xml_schema_uri.text = xml_namespace
+    xml_short_name = xml.etree.ElementTree.SubElement(xml_index, "shortName")
+    xml_short_name.text = short_name
 
     for header in headers:
         if ":" in header:
@@ -335,7 +406,14 @@ def csv_to_search_xml(csv_file, xml_namespace, root_element, title="Metadata Tit
 def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename", export_folder=None,
                additional_namespaces=None):
     """
-    Export the rows of a CSV file as XML metadata documents which can be added to Preservica assets
+        Export the rows of a CSV file as XML metadata documents which can be added to Preservica assets
+
+        :param str csv_file: Path to the csv file
+        :param str xml_namespace: The XML namespace for the created XML documents
+        :param str root_element: The root element for the XML documents
+        :param str file_name_column: The CSV column which should be used to name the xml files
+        :param str export_folder: The path to the export folder
+        :param dict additional_namespaces: A map of prefix, uris to use as additional namespaces
 
     """
     headers = list()
@@ -377,9 +455,237 @@ def cvs_to_xml(csv_file, xml_namespace, root_element, file_name_column="filename
                 yield name
 
 
+def generic_asset_package(preservation_files_dict=None, access_files_dict=None, export_folder=None,
+                          parent_folder=None, compress=True, **kwargs):
+    # some basic validation
+    if export_folder is None:
+        export_folder = tempfile.gettempdir()
+    if not os.path.isdir(export_folder):
+        logger.error("Export Folder Does Not Exist")
+        raise RuntimeError(export_folder, "Export Folder Does Not Exist")
+    if parent_folder is None:
+        logger.error("You must specify a parent folder for the package asset")
+        raise RuntimeError("You must specify a parent folder for the package asset")
+
+    io_ref = None
+    xip = None
+    default_asset_title = None
+    preservation_representation_refs_dict = dict()
+    access_representation_refs_dict = dict()
+
+    security_tag = kwargs.get('SecurityTag', "open")
+    content_type = kwargs.get('CustomType', "")
+
+    if not compress:
+        shutil.register_archive_format("szip", _make_stored_zipfile, None, "UnCompressed ZIP file")
+
+    has_preservation_files = bool((preservation_files_dict is not None) and (len(preservation_files_dict) > 0))
+    has_access_files = bool((access_files_dict is not None) and (len(access_files_dict) > 0))
+
+    if has_preservation_files:
+        if default_asset_title is None:
+            key = list(preservation_files_dict.keys())[0]
+            preservation_files_list = preservation_files_dict[key]
+            default_asset_title = os.path.splitext(os.path.basename(preservation_files_list[0]))[0]
+
+        # create the asset
+        xip, io_ref = __create_io__(file_name=default_asset_title, parent_folder=parent_folder, **kwargs)
+
+    if has_access_files:
+        if default_asset_title is None:
+            key = list(access_files_dict.keys())[0]
+            access_files_list = access_files_dict[key]
+            default_asset_title = os.path.splitext(os.path.basename(access_files_list[0]))[0]
+
+        if io_ref is None:
+            xip, io_ref = __create_io__(file_name=default_asset_title, parent_folder=parent_folder, **kwargs)
+
+    # loop over preservation_files_map
+
+    if has_preservation_files:
+        for representation_name in preservation_files_dict.keys():
+            preservation_files_list = preservation_files_dict[representation_name]
+            preservation_refs_dict = __make_representation_multiple_co__(xip, rep_name=representation_name,
+                                                                         rep_type="Preservation",
+                                                                         rep_files=preservation_files_list,
+                                                                         io_ref=io_ref)
+            preservation_representation_refs_dict[representation_name] = preservation_refs_dict
+
+    if has_access_files:
+        for representation_name in access_files_dict.keys():
+            access_files_list = access_files_dict[representation_name]
+            access_refs_dict = __make_representation_multiple_co__(xip, rep_name=representation_name, rep_type="Access",
+                                                                   rep_files=access_files_list, io_ref=io_ref)
+            access_representation_refs_dict[representation_name] = access_refs_dict
+
+    if has_preservation_files:
+        for representation_name in preservation_representation_refs_dict.keys():
+            preservation_refs_dict = preservation_representation_refs_dict[representation_name]
+            for content_ref, filename in preservation_refs_dict.items():
+                default_content_objects_title = os.path.splitext(os.path.basename(filename))[0]
+
+                preservation_content_title = kwargs.get('Preservation_Content_Title', default_content_objects_title)
+                preservation_content_description = kwargs.get('Preservation_Content_Description',
+                                                              default_content_objects_title)
+
+                if isinstance(preservation_content_title, dict):
+                    preservation_content_title = preservation_content_title.get("filename",
+                                                                                default_content_objects_title)
+
+                if isinstance(preservation_content_description, dict):
+                    preservation_content_description = preservation_content_description.get("filename",
+                                                                                            default_content_objects_title)
+
+                __make_content_objects__(xip, preservation_content_title, content_ref, io_ref, security_tag,
+                                         preservation_content_description, content_type)
+
+    if has_access_files:
+        for representation_name in access_representation_refs_dict.keys():
+            access_refs_dict = access_representation_refs_dict[representation_name]
+            for content_ref, filename in access_refs_dict.items():
+                default_content_objects_title = os.path.splitext(os.path.basename(filename))[0]
+
+                access_content_title = kwargs.get('Access_Content_Title', default_content_objects_title)
+                access_content_description = kwargs.get('Access_Content_Description', default_content_objects_title)
+
+                if isinstance(access_content_title, dict):
+                    access_content_title = access_content_title.get("filename", default_content_objects_title)
+
+                if isinstance(access_content_description, dict):
+                    access_content_description = access_content_title.get("filename", default_content_objects_title)
+
+                __make_content_objects__(xip, access_content_title, content_ref, io_ref, security_tag,
+                                         access_content_description, content_type)
+
+    if has_preservation_files:
+        for representation_name in preservation_representation_refs_dict.keys():
+            preservation_refs_dict = preservation_representation_refs_dict[representation_name]
+            preservation_generation_label = kwargs.get('Preservation_Generation_Label', "")
+            for content_ref, filename in preservation_refs_dict.items():
+                preservation_file_name = os.path.basename(filename)
+                __make_generation__(xip, preservation_file_name, content_ref, preservation_generation_label)
+
+    if has_access_files:
+        for representation_name in access_representation_refs_dict.keys():
+            access_refs_dict = access_representation_refs_dict[representation_name]
+            access_generation_label = kwargs.get('Access_Generation_Label', "")
+            for content_ref, filename in access_refs_dict.items():
+                access_file_name = os.path.basename(filename)
+                __make_generation__(xip, access_file_name, content_ref, access_generation_label)
+
+    if has_preservation_files:
+
+        if 'Preservation_files_fixity_callback' in kwargs:
+            callback = kwargs.get('Preservation_files_fixity_callback')
+        else:
+            callback = Sha1FixityCallBack()
+        for representation_name in preservation_representation_refs_dict.keys():
+            preservation_refs_dict = preservation_representation_refs_dict[representation_name]
+            for content_ref, filename in preservation_refs_dict.items():
+                preservation_file_name = os.path.basename(filename)
+                __make_bitstream__(xip, preservation_file_name, filename, callback)
+
+    if has_access_files:
+
+        if 'Access_files_fixity_callback' in kwargs:
+            callback = kwargs.get('Access_files_fixity_callback')
+        else:
+            callback = Sha1FixityCallBack()
+
+        for representation_name in access_representation_refs_dict.keys():
+            access_refs_dict = access_representation_refs_dict[representation_name]
+            for content_ref, filename in access_refs_dict.items():
+                access_file_name = os.path.basename(filename)
+                __make_bitstream__(xip, access_file_name, filename, callback)
+
+    if 'Identifiers' in kwargs:
+        identifier_map = kwargs.get('Identifiers')
+        for identifier_key, identifier_value in identifier_map.items():
+            if identifier_key:
+                if identifier_value:
+                    identifier = SubElement(xip, 'Identifier')
+                    id_type = SubElement(identifier, "Type")
+                    id_type.text = identifier_key
+                    id_value = SubElement(identifier, "Value")
+                    id_value.text = identifier_value
+                    id_io = SubElement(identifier, "Entity")
+                    id_io.text = io_ref
+
+    if 'Asset_Metadata' in kwargs:
+        metadata_map = kwargs.get('Asset_Metadata')
+        for metadata_ns, metadata_path in metadata_map.items():
+            if metadata_ns:
+                if metadata_path:
+                    if os.path.exists(metadata_path) and os.path.isfile(metadata_path):
+                        descriptive_metadata = xml.etree.ElementTree.parse(source=metadata_path)
+                        metadata = SubElement(xip, 'Metadata', {'schemaUri': metadata_ns})
+                        metadata_ref = SubElement(metadata, 'Ref')
+                        metadata_ref.text = str(uuid.uuid4())
+                        entity = SubElement(metadata, 'Entity')
+                        entity.text = io_ref
+                        content = SubElement(metadata, 'Content')
+                        content.append(descriptive_metadata.getroot())
+
+    if xip is not None:
+        export_folder = export_folder
+        top_level_folder = os.path.join(export_folder, io_ref)
+        os.mkdir(top_level_folder)
+        inner_folder = os.path.join(top_level_folder, io_ref)
+        os.mkdir(inner_folder)
+        os.mkdir(os.path.join(inner_folder, "content"))
+        metadata_path = os.path.join(inner_folder, "metadata.xml")
+        metadata = open(metadata_path, "wt", encoding='utf-8')
+        metadata.write(prettify(xip))
+        metadata.close()
+        for representation_name in preservation_representation_refs_dict.keys():
+            preservation_refs_dict = preservation_representation_refs_dict[representation_name]
+            for content_ref, filename in preservation_refs_dict.items():
+                src_file = filename
+                dst_file = os.path.join(os.path.join(inner_folder, "content"), os.path.basename(filename))
+                shutil.copyfile(src_file, dst_file)
+        for representation_name in access_representation_refs_dict.keys():
+            access_refs_dict = access_representation_refs_dict[representation_name]
+            for content_ref, filename in access_refs_dict.items():
+                src_file = filename
+                dst_file = os.path.join(os.path.join(inner_folder, "content"), os.path.basename(filename))
+                shutil.copyfile(src_file, dst_file)
+        if compress:
+            shutil.make_archive(top_level_folder, 'zip', top_level_folder)
+        else:
+            shutil.make_archive(top_level_folder, 'szip', top_level_folder)
+        shutil.rmtree(top_level_folder)
+        return top_level_folder + ".zip"
+
+
 def complex_asset_package(preservation_files_list=None, access_files_list=None, export_folder=None,
-                          parent_folder=None, **kwargs):
+                          parent_folder=None, compress=True, **kwargs):
     """
+
+            Create a Preservica package containing a single Asset from a multiple preservation files
+            and optional access files.
+            The Asset contains multiple Content Objects within each representation.
+
+            If only the preservation files are provided the asset has one representation
+
+
+            :param list preservation_files_list: Paths to the preservation files
+            :param list access_files_list: Paths to the access files
+            :param str export_folder: The package location folder
+            :param Folder parent_folder: The folder to ingest the asset into
+            :param bool compress: Compress the ZIP file
+            :param str Title: Asset Title
+            :param str Description: Asset Description
+            :param str SecurityTag: Asset SecurityTag
+            :param str CustomType: Asset CustomType
+            :param str Preservation_Content_Title: Title of the Preservation Representation Content Object
+            :param str Preservation_Content_Description: Description of the Preservation Representation Content Object
+            :param str Access_Content_Title: Title of the Access Representation Content Object
+            :param str Access_Content_Description: Description of the Access Representation Content Object
+            :param dict Asset_Metadata: Dictionary of Asset metadata documents
+            :param dict Identifiers: Dictionary of Asset rd party identifiers
+
+
+
         optional kwargs map
         'Title'                                 Asset Title
         'Description'                           Asset Description
@@ -396,6 +702,8 @@ def complex_asset_package(preservation_files_list=None, access_files_list=None,
         'Preservation_files_fixity_callback'    Callback to allow external generated fixity values
         'Access_files_fixity_callback'          Callback to allow external generated fixity values
         'IO_Identifier_callback'                Callback to allow external generated Asset identifier
+        'Preservation_Representation_Name'      Name of the Preservation Representation
+        'Access_Representation_Name'            Name of the Access Representation
     """
     # some basic validation
     if export_folder is None:
@@ -416,6 +724,9 @@ def complex_asset_package(preservation_files_list=None, access_files_list=None,
     security_tag = kwargs.get('SecurityTag', "open")
     content_type = kwargs.get('CustomType', "")
 
+    if not compress:
+        shutil.register_archive_format("szip", _make_stored_zipfile, None, "UnCompressed ZIP file")
+
     has_preservation_files = bool((preservation_files_list is not None) and (len(preservation_files_list) > 0))
     has_access_files = bool((access_files_list is not None) and (len(access_files_list) > 0))
 
@@ -435,12 +746,16 @@ def complex_asset_package(preservation_files_list=None, access_files_list=None,
 
     if has_preservation_files:
         # add the content objects
-        preservation_refs_dict = __make_representation_multiple_co__(xip, "Preservation", "Preservation",
-                                                                     preservation_files_list, io_ref)
+        representation_name = kwargs.get('Preservation_Representation_Name', "Preservation")
+        preservation_refs_dict = __make_representation_multiple_co__(xip, rep_name=representation_name,
+                                                                     rep_type="Preservation",
+                                                                     rep_files=preservation_files_list, io_ref=io_ref)
 
     if has_access_files:
         # add the content objects
-        access_refs_dict = __make_representation_multiple_co__(xip, "Access", "Access", access_files_list, io_ref)
+        access_name = kwargs.get('Access_Representation_Name', "Access")
+        access_refs_dict = __make_representation_multiple_co__(xip, rep_name=access_name, rep_type="Access",
+                                                               rep_files=access_files_list, io_ref=io_ref)
 
     if has_preservation_files:
 
@@ -561,28 +876,43 @@ def complex_asset_package(preservation_files_list=None, access_files_list=None,
             src_file = filename
             dst_file = os.path.join(os.path.join(inner_folder, "content"), os.path.basename(filename))
             shutil.copyfile(src_file, dst_file)
-        shutil.make_archive(top_level_folder, 'zip', top_level_folder)
+        if compress:
+            shutil.make_archive(top_level_folder, 'zip', top_level_folder)
+        else:
+            shutil.make_archive(top_level_folder, 'szip', top_level_folder)
         shutil.rmtree(top_level_folder)
         return top_level_folder + ".zip"
 
 
-def simple_asset_package(preservation_file=None, access_file=None, export_folder=None, parent_folder=None, **kwargs):
+def simple_asset_package(preservation_file=None, access_file=None, export_folder=None, parent_folder=None,
+                         compress=True, **kwargs):
     """
-        optional kwargs map
-        'Title'                             Asset Title
-        'Description'                       Asset Description
-        'SecurityTag'                       Asset Security Tag
-        'CustomType'                        Asset Type
-        'Preservation_Content_Title'        Content Object Title of the Preservation Object
-        'Preservation_Content_Description'  Content Object Description of the Preservation Object
-        'Access_Content_Title'              Content Object Title of the Access Object
-        'Access_Content_Description'        Content Object Description of the Access Object
-        'Preservation_Generation_Label'     Generation Label for the Preservation Object
-        'Access_Generation_Label'           Generation Label for the Access Object
-        'Asset_Metadata'                    Map of metadata schema/documents to add to asset
-        'Identifiers'                       Map of asset identifiers
-        'Preservation_files_fixity_callback'    Callback to allow external generated fixity values
-        'Access_files_fixity_callback'    Callback to allow external generated fixity values
+            Create a Preservica package containing a single Asset from a single preservation file
+            and an optional access file.
+            The Asset contains one Content Object for each representation.
+
+            If only the preservation file is provided the asset has one representation
+
+
+            :param str preservation_file: Path to the preservation file
+            :param str access_file: Path to the access file
+            :param str export_folder: The package location folder
+            :param Folder parent_folder: The folder to ingest the asset into
+            :param bool compress: Compress the ZIP file
+            :param str Title: Asset Title
+            :param str Description: Asset Description
+            :param str SecurityTag: Asset SecurityTag
+            :param str CustomType: Asset CustomType
+            :param str Preservation_Content_Title: Title of the Preservation Representation Content Object
+            :param str Preservation_Content_Description: Description of the Preservation Representation Content Object
+            :param str Access_Content_Title: Title of the Access Representation Content Object
+            :param str Access_Content_Description: Description of the Access Representation Content Object
+            :param dict Asset_Metadata: Dictionary of Asset metadata documents
+            :param dict Identifiers: Dictionary of Asset rd party identifiers
+
+
+
+
     """
 
     # some basic validation
@@ -605,7 +935,11 @@ def simple_asset_package(preservation_file=None, access_file=None, export_folder
         access_file_list.append(access_file)
 
     return complex_asset_package(preservation_files_list=preservation_file_list, access_files_list=access_file_list,
-                                 export_folder=export_folder, parent_folder=parent_folder, **kwargs)
+                                 export_folder=export_folder, parent_folder=parent_folder, compress=compress, **kwargs)
+
+
+def upload_config():
+    return transfer_config
 
 
 class UploadAPI(AuthenticatedAPI):
@@ -613,6 +947,20 @@ class UploadAPI(AuthenticatedAPI):
     def ingest_twitter_feed(self, twitter_user=None, num_tweets: int = 25, twitter_consumer_key=None,
                             twitter_secret_key=None, folder=None, callback=None, **kwargs):
 
+        """
+            Ingest tweets from a twitter stream by twitter username
+
+            :param str twitter_user: Twitter Username
+            :param int num_tweets: The number of tweets from the stream
+            :param str twitter_consumer_key: Optional asset title
+            :param str twitter_secret_key: Optional asset description
+            :param str folder: Folder to ingest into
+            :param callback callback: Optional upload progress callback
+            :raises RuntimeError:
+
+
+        """
+
         def get_image(m, has_video_element):
             media_url_https_ = m["media_url_https"]
             if media_url_https_:
@@ -780,6 +1128,21 @@ class UploadAPI(AuthenticatedAPI):
                 sleep(2)
 
     def ingest_web_video(self, url=None, parent_folder=None, **kwargs):
+        """
+            Ingest a web video such as YouTube etc based on the URL
+
+            :param str url: URL to the youtube video
+            :param Folder parent_folder: The folder to ingest the video into
+            :param str Title: Optional asset title
+            :param str Description: Optional asset description
+            :param str SecurityTag: Optional asset security tag
+            :param dict Identifiers: Optional asset 3rd party identifiers
+            :param dict Asset_Metadata: Optional asset additional descriptive metadata
+            :param callback callback: Optional upload progress callback
+            :raises RuntimeError:
+
+
+        """
         try:
             import youtube_dl
         except ImportError:
@@ -797,7 +1160,7 @@ class UploadAPI(AuthenticatedAPI):
             'progress_hooks': [my_hook],
         }
 
-        #if True:
+        # if True:
         #    ydl_opts['writesubtitles'] = True
         #    ydl_opts['writeautomaticsub'] = True
         #    ydl_opts['subtitleslangs'] = ['en']
@@ -849,6 +1212,22 @@ class UploadAPI(AuthenticatedAPI):
             self.upload_zip_package(path_to_zip_package=package, folder=parent_folder, callback=callback)
 
     def upload_zip_package(self, path_to_zip_package, folder=None, callback=None, delete_after_upload=False):
+        """
+        Uploads a zip file package and starts an ingest workflow
+
+        :param str path_to_zip_package: Path to the package
+        :param Folder folder: The folder to ingest the package into
+        :param str callback: Optional callback to allow the callee to monitor the upload progress
+        :param bool delete_after_upload: Delete the local copy of the package after the upload has completed
+
+        :return: preservica-progress-token to allow the workflow progress to be monitored
+        :rtype: str
+
+
+        :raises RuntimeError:
+
+
+        """
         bucket = f'{self.tenant.lower()}.package.upload'
         endpoint = f'https://{self.server}/api/s3/buckets'
         self.token = self.__token__()
@@ -866,36 +1245,20 @@ class UploadAPI(AuthenticatedAPI):
         if os.path.exists(path_to_zip_package) and os.path.isfile(path_to_zip_package):
             try:
                 key_id = str(uuid.uuid4()) + ".zip"
-                s3_client.upload_file(path_to_zip_package, bucket, key_id, ExtraArgs=metadata,
-                                      Callback=callback, Config=transfer_config)
-                if delete_after_upload:
-                    os.remove(path_to_zip_package)
-            except ClientError as e:
-                raise e
 
-    def upload_zip_package_progress_token(self, path_to_zip_package, folder=None, delete_after_upload=False):
-        bucket = f'{self.tenant.lower()}.package.upload'
-        endpoint = f'https://{self.server}/api/s3/buckets'
-        self.token = self.__token__()
-        s3_client = boto3.client('s3', endpoint_url=endpoint, aws_access_key_id=self.token,
-                                 aws_secret_access_key="NOT_USED",
-                                 config=Config(s3={'addressing_style': 'path'}))
+                transfer = S3Transfer(client=s3_client, config=transfer_config)
 
-        metadata = dict()
-        if folder is not None:
-            if hasattr(folder, "reference"):
-                metadata = {'structuralobjectreference': folder.reference}
-            elif isinstance(folder, str):
-                metadata = {'structuralobjectreference': folder}
+                transfer.PutObjectTask = PutObjectTask
+                transfer.CompleteMultipartUploadTask = CompleteMultipartUploadTask
+                transfer.upload_file = upload_file
 
-        if os.path.exists(path_to_zip_package) and os.path.isfile(path_to_zip_package):
-            try:
-                key_id = str(uuid.uuid4()) + ".zip"
-                with open(path_to_zip_package, 'rb') as fd:
-                    response = s3_client.put_object(Body=fd, Bucket=bucket, Key=key_id, Metadata=metadata)
+                response = transfer.upload_file(self=transfer, filename=path_to_zip_package, bucket=bucket, key=key_id,
+                                                extra_args=metadata, callback=callback)
 
                 if delete_after_upload:
                     os.remove(path_to_zip_package)
+
                 return response['ResponseMetadata']['HTTPHeaders']['preservica-progress-token']
+
             except ClientError as e:
                 raise e
